
#include "LatticeConfig.h"
#include "lattice.h"

#include "EventHandler.h"
#include "GuiWrapper.h"

#include "SDL.h"

#include <iostream>



Lattice lattice;
EventHandler handler(&lattice);
GuiWrapper gui(&handler);

unsigned int oldTime = 0;

bool InitSDL();
void RenderScene();
void ResizeWindow(int width, int height);

 

int main (int argc, char *argv[]) {

	bool done = false;
	int videoFlags;
	int screenWidth  = 640;
	int screenHeight = 480;
	int screenBpp    = 32;
	const SDL_VideoInfo* info = 0;
	const char appName[] = "Lattice Demo";
	std::string DataDir = LATTICE_DATA_DIR;
	SDL_Event event;
	SDL_Surface *surface;
	std::string userInput = "";

	while (userInput != "y" && userInput != "n") {
		std::cout << "Fullscreen mode? (y/n): ";
		std::cin >> userInput;
	}

	//init sdl and sdl_net
	if ( !InitSDL() ) {
		printf("Video initialization failed.\n");
		exit(0);
	}

	//call SDL_Quit if program exits
	atexit(SDL_Quit);

	// Get video information
    info = SDL_GetVideoInfo( );

    if( !info ) {
        printf("Video query failed: %s\n", SDL_GetError());
		exit(0);        
    }

    screenBpp = info->vfmt->BitsPerPixel;


	// Request minimum values
    SDL_GL_SetAttribute( SDL_GL_RED_SIZE, 5 );
    SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, 5 );
    SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, 5 );
    SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, 16 );
    SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );

	//set window title
	SDL_WM_SetCaption(appName, appName);

	//set video flags
	videoFlags = SDL_OPENGL;  
	//videoFlags |= SDL_RESIZABLE;
	if (userInput == "y") {
		screenWidth = 1024;
		screenHeight = 768;
		videoFlags |= SDL_FULLSCREEN;
	}
	// get a SDL surface
	surface = SDL_SetVideoMode( screenWidth, screenHeight, screenBpp,
				videoFlags );

	//check that we got a surface
	if ( !surface ) {
		printf("Video mode set failed: %s\n", SDL_GetError( ) );
		exit(0);
	}

	//initialize gui
	if ( !gui.Init(DataDir.c_str()) ) {
		printf("Failed to initialize gui.\n");
		exit(0);
	}

	//initialize lattice
	lattice.hack_init(screenWidth, screenHeight);

	//initialize window size
	ResizeWindow( screenWidth, screenHeight );

	//--- main loop ----

	while ( !done ) {

	    //handle the events in the queue 
	    while ( SDL_PollEvent( &event ) )
		{
		    switch( event.type )
			{
			    
			case SDL_VIDEORESIZE:
				surface = SDL_SetVideoMode( event.resize.w, event.resize.h, screenBpp, videoFlags );
				if ( !surface ) {
					printf("Could not get a surface after resize: %s\n", SDL_GetError() );
					exit(0);
				}

				ResizeWindow( event.resize.w, event.resize.h );
				RenderScene();
				break;
			case SDL_KEYDOWN:
				gui.AsciiKeyDown(event.key.keysym.sym);
				break;
			case SDL_MOUSEMOTION:
				gui.MouseMoved(event.motion.x, event.motion.y); //event.motion.xrel, event.motion.yrel
				break;
			case SDL_MOUSEBUTTONDOWN:
				if (event.button.button == SDL_BUTTON_LEFT)
					gui.MouseClicked(1, true, event.button.x, event.button.y);
				else if (event.button.button == SDL_BUTTON_RIGHT)
					gui.MouseClicked(2, true, event.button.x, event.button.y);
				break;
			case SDL_MOUSEBUTTONUP:
				if (event.button.button == SDL_BUTTON_LEFT)
					gui.MouseClicked(1, false, event.button.x, event.button.y);
				else if (event.button.button == SDL_BUTTON_RIGHT)
					gui.MouseClicked(2, false, event.button.x, event.button.y);
				break;
			case SDL_QUIT:
				done = true;
				break;
			default:
				break;
			}
		}

		if (!done) {

			//render
			RenderScene();
		}
	}

	SDL_Quit();

	return 0;
}

bool InitSDL() {

	// initialize SDL
	if(SDL_Init( SDL_INIT_VIDEO ) < 0)
	{
		printf("SDL_Init: %s\n",SDL_GetError());
		return false;
	}

	//turn on keyboard key repeat
	if ( ( SDL_EnableKeyRepeat( 500, SDL_DEFAULT_REPEAT_INTERVAL ) ) ) {
		printf("Setting keyboard repeat failed: %s\n", SDL_GetError( ) );
		return false;
	}

	return true;
}

void ResizeWindow(int width, int height) {

  gui.SetDimensions(width, height);
  lattice.hack_reshape(width, height);

}

void RenderScene()
{
	glClearColor(handler.red, handler.green, handler.blue, 0.0f);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	//get elapsed milliseconds
	int time = SDL_GetTicks();
	double dt = (time-oldTime)/1000.0f;


	//save attributes
	glPushAttrib(GL_ALL_ATTRIB_BITS);

		lattice.hack_draw(dt);

		// save currently loaded texture
		int savedTexture = 0;
		glGetIntegerv(GL_TEXTURE_BINDING_2D,(GLint*)&savedTexture);

	//pop attributes
	glPopAttrib();

	gui.Update(dt);

	glPushAttrib(GL_ALL_ATTRIB_BITS);

		glDisable (GL_COLOR_MATERIAL);
		glEnable(GL_BLEND);
		glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
		glColor4f(1,1,1,1);
		glDisable(GL_TEXTURE_2D);

		if (gui.IsActive()) gui.RenderScene();

	glPopAttrib();
	glBindTexture (GL_TEXTURE_2D, savedTexture);

	oldTime = time;
	SDL_GL_SwapBuffers( );

}





