
#include "LightsConfig.h"
#include "Lights.h"

#include "EventHandler.h"
#include "GuiWrapper.h"

#include "SDL.h"

#include <iostream>


Lights lights;
EventHandler handler(&lights);
GuiWrapper gui(&handler, &lights);

unsigned int oldTime = 0;

bool InitSDL();
void RenderScene();
void ResizeWindow(int width, int height);

 

int main (int argc, char *argv[]) {

	bool done = false;
	int videoFlags;
	int screenWidth  = 800;
	int screenHeight = 600;
	int screenBpp    = 32;
	const SDL_VideoInfo* info = 0;
	const char appName[] = "Lights Demo";
	std::string DataDir = LIGHTS_DATA_DIR;
	SDL_Event event;
	SDL_Surface *surface;
	std::string userInput = "";

	while (userInput != "y" && userInput != "n") {
		std::cout << "Fullscreen mode? (y/n): ";
		std::cin >> userInput;
	}

	//init sdl and sdl_net
	if ( !InitSDL() ) {
		printf("Video initialization failed.\n");
		exit(0);
	}

	//call SDL_Quit if program exits
	atexit(SDL_Quit);

	// Get video information
    info = SDL_GetVideoInfo( );

    if( !info ) {
        printf("Video query failed: %s\n", SDL_GetError());
		exit(0);        
    }

    screenBpp = info->vfmt->BitsPerPixel;

	// Request minimum values
    SDL_GL_SetAttribute( SDL_GL_RED_SIZE, 5 );
    SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, 5 );
    SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, 5 );
    SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, 16 );
    SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );

	//set window title
	SDL_WM_SetCaption(appName, appName);

	//set video flags
	videoFlags = SDL_OPENGL;  
	//videoFlags |= SDL_RESIZABLE;
	if (userInput == "y") {
		screenWidth = 1024;
		screenHeight = 768;
		videoFlags |= SDL_FULLSCREEN;
	}
	// get a SDL surface
	surface = SDL_SetVideoMode( screenWidth, screenHeight, screenBpp,
				videoFlags );

	//check that we got a surface
	if ( !surface ) {
		printf("Video mode set failed: %s\n", SDL_GetError( ) );
		exit(0);
	}

	//initialize gui
	if ( !gui.Init(DataDir.c_str()) ) {
		printf("Failed to initialize gui.\n");
		exit(0);
	}

	//initialize lights
	lights.Init();

	//initialize window size
	ResizeWindow( screenWidth, screenHeight );

	//--- main loop ----

	while ( !done ) {

	    //handle the events in the queue 
	    while ( SDL_PollEvent( &event ) )
		{
		    switch( event.type )
			{
			    
			case SDL_VIDEORESIZE:
				surface = SDL_SetVideoMode( event.resize.w, event.resize.h, screenBpp, videoFlags );
				if ( !surface ) {
					printf("Could not get a surface after resize: %s\n", SDL_GetError() );
					exit(0);
				}

				ResizeWindow( event.resize.w, event.resize.h );
				RenderScene();
				break;
			case SDL_KEYDOWN:
				gui.AsciiKeyDown(event.key.keysym.sym);
				break;
			case SDL_MOUSEMOTION:
				gui.MouseMoved(event.motion.x, event.motion.y); //event.motion.xrel, event.motion.yrel
				lights.MouseMove(event.motion.xrel, event.motion.yrel);
				break;
			case SDL_MOUSEBUTTONDOWN:
				if (event.button.button == SDL_BUTTON_LEFT) {
					if (!gui.MouseClicked(1, true, event.button.x, event.button.y)) {
						lights.MouseDown();
					}
				}
				else if (event.button.button == SDL_BUTTON_RIGHT)
					gui.MouseClicked(2, true, event.button.x, event.button.y);
				break;
			case SDL_MOUSEBUTTONUP:
				if (event.button.button == SDL_BUTTON_LEFT) {
					gui.MouseClicked(1, false, event.button.x, event.button.y);
					lights.MouseUp();
				}
				else if (event.button.button == SDL_BUTTON_RIGHT)
					gui.MouseClicked(2, false, event.button.x, event.button.y);
				break;
			case SDL_QUIT:
				done = true;
				break;
			default:
				break;
			}
		}

		if (!done) {

			//render
			RenderScene();
		}
	}

	SDL_Quit();

	return 0;
}

bool InitSDL() {

	// initialize SDL
	if(SDL_Init( SDL_INIT_VIDEO ) < 0)
	{
		printf("SDL_Init: %s\n",SDL_GetError());
		return false;
	}

	//turn on keyboard key repeat
	if ( ( SDL_EnableKeyRepeat( 500, SDL_DEFAULT_REPEAT_INTERVAL ) ) ) {
		printf("Setting keyboard repeat failed: %s\n", SDL_GetError( ) );
		return false;
	}

	return true;
}

void ResizeWindow(int width, int height) {

  float aspectRatio = 1.33f;
  height            = height <= 0 ? 1 : height;
  aspectRatio       = (float)width/height;

  glViewport(0, 0, width, height);
  glMatrixMode(GL_PROJECTION);

  glLoadIdentity();
  gluPerspective( 45.0f, aspectRatio, 0.1f, 100.0f );

  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();

  gui.SetDimensions(width, height);


}

void RenderScene()
{
	//get elapsed milliseconds
	int time = SDL_GetTicks();
	double dt = (time-oldTime)/1000.0f;


	//save attributes
	glPushAttrib(GL_ALL_ATTRIB_BITS);

		lights.Draw();

	//pop attributes
	glPopAttrib();

	gui.Update(dt);

	glPushAttrib(GL_ALL_ATTRIB_BITS);

		glEnable(GL_BLEND);
		glDisable(GL_TEXTURE_2D);

		if (gui.IsActive()) gui.RenderScene();

	glPopAttrib();

	oldTime = time;
	SDL_GL_SwapBuffers( );

}





